@@ -20,9 +20,9 @@
 """Manages the default speech server for orca.  A script can use this
 as its speech server, or it can feel free to create one of its own."""
 
-__id__        = "$Id: speech.py 4433 2009-01-19 15:46:14Z wwalker $"
-__version__   = "$Revision: 4433 $"
-__date__      = "$Date: 2009-01-19 10:46:14 -0500 (Mon, 19 Jan 2009) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -38,6 +38,7 @@ import keynames
 import orca
 import orca_state
 import settings
+import speech_generator
 
 from acss import ACSS
 from orca_i18n import _           # for gettext support
@@ -48,10 +49,9 @@ _speechserver = None
 
 # regular expressions for multiCaseStrings
 #
-multiCaseReg1 = re.compile("([a-z]+)([A-Z][a-z]+)")
-multiCaseReg2 = re.compile("([a-z]+)([A-Z]+)")
-multiCaseReg3 = re.compile("([A-Z]{2}[A-Z]+)([a-z]+)")
-multiCaseReg4 = re.compile("([A-Z])([A-Z][a-z]+)")
+multiCaseReg1 = re.compile("([a-z]+)([A-Z])")
+multiCaseReg2 = re.compile("([A-Z][A-Z]+)([A-Z][a-z]+)")
+multiCaseReg3 = re.compile("([A-Z])([A-Z][a-z]+)")
 
 def getSpeechServerFactories():
     """Imports all known SpeechServer factory modules.  Returns a list
@@ -138,20 +138,35 @@ def sayAll(utteranceIterator, progressCallback):
             debug.println(debug.LEVEL_INFO, logLine)
             log.info(logLine)
 
-def speak(text, acss=None, interrupt=True):
-    """Speaks all queued text immediately.  If text is not None,
-    it is added to the queue before speaking.
+def _speak(text, acss, interrupt):
+    """Speaks the individual string using the given ACSS."""
 
-    Arguments:
-    - text:      optional text to add to the queue before speaking
-    - acss:      acss.ACSS instance; if None,
-                 the default voice settings will be used.
-                 Otherwise, the acss settings will be
-                 used to augment/override the default
-                 voice settings.
-    - interrupt: if True, stops any speech in progress before
-                 speaking the text
-    """
+    if settings.speakMultiCaseStringsAsWords:
+        text = _processMultiCaseString(text)
+    if orca_state.activeScript and orca_state.usePronunciationDictionary:
+        text = orca_state.activeScript.adjustForPronunciation(text)
+    if settings.speakMultiCaseStringsAsWords:
+        text = _processMultiCaseString(text)
+
+    logLine = "SPEECH OUTPUT: '" + text + "'"
+    extraDebug = ""
+    if acss in settings.voices.values():
+        for key in settings.voices:
+            if acss == settings.voices[key]:
+                if key != settings.DEFAULT_VOICE:
+                    extraDebug = " voice=%s" % key
+                break
+    debug.println(debug.LEVEL_INFO, logLine + extraDebug)
+    log.info(logLine + extraDebug)
+
+    if _speechserver:
+        _speechserver.speak(text, __resolveACSS(acss), interrupt)
+
+
+def speak(content, acss=None, interrupt=True):
+    """Speaks the given content.  The content can be either a simple
+    string or an array of arrays of objects returned by a speech
+    generator."""
 
     # We will not interrupt a key echo in progress.
     #
@@ -162,19 +177,45 @@ def speak(text, acss=None, interrupt=True):
     if settings.silenceSpeech:
         return
 
-    if settings.speakMultiCaseStringsAsWords:
-        text = _processMultiCaseString(text)
-    if orca_state.activeScript and orca_state.usePronunciationDictionary:
-        text = orca_state.activeScript.adjustForPronunciation(text)
-    if settings.speakMultiCaseStringsAsWords:
-        text = _processMultiCaseString(text)
+    if isinstance(content, basestring):
+        subString = content
+    elif isinstance(content, list):
+        subString = None
+        for element in content:
+            if isinstance(element, basestring):
+                if subString:
+                    subString += " " + element
+                else:
+                    subString = element
+            elif isinstance(element, speech_generator.Pause):
+                if subString:
+                    if subString[-1] != ".":
+                        subString += "."
+                    _speak(subString, acss, interrupt)
+                    subString = None
+            elif isinstance(element, speech_generator.LineBreak):
+                if subString:
+                    _speak(subString, acss, interrupt)
+                    subString = None
+            else:
+                if subString:
+                    _speak(subString, acss, interrupt)
+                subString = None
+                if isinstance(element, list):
+                    speak(element, acss, interrupt)
+                elif isinstance(element, ACSS):
+                    acss = ACSS(acss)
+                    acss.update(element)
+                else:
+                    debug.println(debug.LEVEL_WARNING,
+                                  "UNKNOWN speech element: '%s'" % element)
+    else:
+        debug.printStack(debug.LEVEL_WARNING)
+        debug.println(debug.LEVEL_WARNING, 
+                      "bad content send to speech.speak: '%s'", repr(content))
 
-    logLine = "SPEECH OUTPUT: '" + text + "'"
-    debug.println(debug.LEVEL_INFO, logLine)
-    log.info(logLine)
-
-    if _speechserver:
-        _speechserver.speak(text, __resolveACSS(acss), interrupt)
+    if subString:
+        _speak(subString, acss, interrupt)
 
 def speakKeyEvent(event_string, eventType):
     """Speaks a key event immediately.
@@ -239,49 +280,6 @@ def isSpeaking():
     else:
         return False
 
-def speakUtterances(utterances, acss=None, interrupt=True):
-    """Speaks the given list of utterances immediately.
-
-    Arguments:
-    - list:      list of strings to be spoken
-    - acss:      acss.ACSS instance; if None,
-                 the default voice settings will be used.
-                 Otherwise, the acss settings will be
-                 used to augment/override the default
-                 voice settings.
-    - interrupt: if True, stop any speech currently in progress.
-    """
-
-    # We will not interrupt a key echo in progress.
-    #
-    if orca_state.lastKeyEchoTime:
-        interrupt = interrupt \
-            and ((time.time() - orca_state.lastKeyEchoTime) > 0.5)
-
-    if settings.silenceSpeech:
-        return
-    i = 0
-    length = len(utterances)
-    while ( i < length ):
-        utterance = utterances[i]
-        if settings.speakMultiCaseStringsAsWords:
-            utterance = _processMultiCaseString(utterance)
-        if settings.speakMultiCaseStringsAsWords:
-            utterances[i] = _processMultiCaseString(utterance)
-            utterance = utterances[i] 
-        if settings.speakMultiCaseStringsAsWords:
-            utterance = _processMultiCaseString(utterance)
-        i = i + 1
-        
-        logLine = "SPEECH OUTPUT: '" + utterance + "'"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
-
-    if _speechserver:
-        _speechserver.speakUtterances(utterances,
-                                       __resolveACSS(acss),
-                                       interrupt)
-
 def getInfo():
     info = None
     if _speechserver:
@@ -380,5 +378,4 @@ def _processMultiCaseString(string):
     string = multiCaseReg1.sub('\\1 \\2', string)
     string = multiCaseReg2.sub('\\1 \\2', string)
     string = multiCaseReg3.sub('\\1 \\2', string)    
-    string = multiCaseReg4.sub('\\1 \\2', string)
     return string
